require 'installer/action'
require 'installer/errors'

class Installer::Planner::Step::BackupLogs < Installer::Planner::Step
  def initialize(infra, data={}, pre=nil, post=nil)
    super

    @stop_on_fail = true

    @log_dir = '/var/log/pe-installer'
    @local_installer = '/opt/puppet/share/installer'
    @local_inst = @local_installer + '/installer'
    @local_ans = @local_installer + '/answers'
    @installer_etc = '/etc/puppetlabs/installer'
  end

  # Call from executor with a block to handle the actions.
  def perform_step(&execution_block)
    @infra.hosts.each do |hostname, host|
      # Create temp log dir
      mktmpdir_cmd = "mktemp -d -t pe-installer-log-XXXXX"
      mktmpdir = Installer::Action::Execute.new(:hosts => [host], :cmd => mktmpdir_cmd )
      yield mktmpdir
      unless mktmpdir.success?
        error(@host.hostname, "Failed to create a temporary directory for the installer log files: #{mktmpdir.error_message(hostname)}.")
        next
      end

      tmpdir = mktmpdir.results[hostname][:stdout].strip

      # Chmod tmp dir
      tmpdir_chmod_cmd = "chmod 700 '#{tmpdir}'"
      chmodtmpdir = Installer::Action::Execute.new(:hosts => [host], :sudo => true, :cmd => tmpdir_chmod_cmd)
      yield chmodtmpdir
      unless chmodtmpdir
        error(hostname, "Failed to execute '#{tmpdir_chmod_cmd}'. #{chmodtmpdir.error_message(hostname)}")
        next
      end

      # Copy log file
      local_installer_log = "#{@log_dir}/installer.log"
      tmp_installer_log = "#{tmpdir}/installer.log"
      copylogs = Installer::Action::Upload.new(:to_host => host, :content => File.read(local_installer_log), :to => tmp_installer_log)
      yield copylogs
      next unless copylogs.success?

      # Copy over to log on remote
      date = DateTime.now.rfc3339
      remote_installer_log = "#{@log_dir}/installer-#{date}.log"
      remotecopylogs = Installer::Action::Execute.new(:hosts => [host], :sudo => true, :cmd => "cp #{tmp_installer_log} #{remote_installer_log}" )
      yield remotecopylogs
      unless remotecopylogs.success?
        error(hostname, "Failed to copy '#{tmp_installer_log}' to '#{remote_installer_log}'. #{remotecopylogs.error_message(hostname)}")
        next
      end

      # Chmod .json
      installer_chmod_cmd = "chmod 600 #{remote_installer_log}"
      chmodlog = Installer::Action::Execute.new(:hosts => [host], :sudo => true, :cmd => installer_chmod_cmd)
      yield chmodlog
      unless chmodlog.success?
        error(hostname, "Failed to execute '#{installer_chmod_cmd}'. #{chmodlog.error_message(hostname)}")
        next
      end

      # Copy Installer details.json
      tmp_details_json = "#{tmpdir}/details.json"
      local_details_json = "#{@local_ans}/details.json"
      copydetails = Installer::Action::Upload.new(:to_host => host, :from => local_details_json, :to => tmp_details_json)
      yield copydetails
      return unless copydetails.success?

      # Copy details.json to /etc/puppetlabs
      remote_details_json = "#{@installer_etc}/details.json"
      remotecopydetails = Installer::Action::Execute.new(:hosts => [host], :sudo => true, :cmd => "cp #{tmp_details_json} #{remote_details_json}")
      yield remotecopydetails
      unless remotecopydetails.success?
        error(hostname, "Failed to copy '#{tmp_details_json}' to '#{remote_details_json}'. #{remotecopydetails.error_message(hostname)}")
        next
      end

      # Chmod details.json
      details_chmod_cmd = "chmod 600 #{@installer_etc}/details.json"
      chmoddetails = Installer::Action::Execute.new(:hosts => [host], :sudo => true, :cmd => details_chmod_cmd)
      yield chmoddetails
      unless chmoddetails.success?
        error(hostname, "Failed to execute '#{details_chmod_cmd}'.")
        next
      end

      # Destroy temp log dir
      rmtmpdir = Installer::Action::Execute.new(:hosts => [host], :cmd => "rm -rf #{tmpdir}")
      yield rmtmpdir
      unless rmtmpdir.success?
        error(hostname, "Failed to remove temporary directory '#{tmpdir}'. #{rmtmpdir.error_message(hostname)}")
        next
      end
    end
  end

  def name
    :backup_logs
  end

  def description
    "Backing up installer log files to #{@infra.hosts.values.join(',')}."
  end
end
